from transformers import AutoTokenizer
from bigdl.llm.transformers import AutoModel
import torch

model_path = r"D:\Code\chatglm2-6b"

model = AutoModel.from_pretrained(model_path, load_in_4bit=True, trust_remote_code=True)
tokenizer = AutoTokenizer.from_pretrained(model_path, trust_remote_code=True)

CHATGLM_V2_PROMPT_TEMPLATE = "问：{prompt}\n\n答："

while True:
    user_input = input("用户：")  # 用户输入问题
    conversation_history = [user_input]  # 清空历史记录，只保留当前用户输入

    # 将整个对话历史作为模型输入
    conversation_text = '\n'.join([CHATGLM_V2_PROMPT_TEMPLATE.format(prompt=utterance) for utterance in conversation_history])
    input_ids = tokenizer.encode(conversation_text, return_tensors="pt")

    # 生成答案
    with torch.inference_mode():
        output = model.generate(input_ids, max_length=1024)
        model_response = tokenizer.decode(output[0], skip_special_tokens=True)

    # 提取模型的回答
    model_response = model_response.replace(CHATGLM_V2_PROMPT_TEMPLATE, "").strip()

    print(f"模型：{model_response}")
