import torch

from transformers import AutoTokenizer
from bigdl.llm.transformers import AutoModel
from bigdl.llm.optimize import load_low_bit, low_memory_init

with low_memory_init():
    model= AutoModel.from_pretrained("chatglm2-6b", trust_remote_code=True)
model_path = "chatglm2-6b_lowbit"
model = load_low_bit(model, model_path)
tokenizer = AutoTokenizer.from_pretrained("chatglm2-6b", trust_remote_code=True)

CHATGLM_V2_PROMPT_TEMPLATE = "问：{prompt}\n\n答："

conversation_history = []  # 存储聊天历史
Identity="你的身份是一个课程助教，你叫VidBot"
conversation_history.append(Identity.strip())

def chat_handler(websocket, path):

    print("WebSocket连接已建立")
    while True:
        user_input=input()
        print(f"用户：{user_input}")  # 打印用户输入

        # 将用户输入添加到聊天历史
        conversation_history.append(user_input)

        model_response = generate_response(conversation_history)
        return model_response

def generate_response(conversation_history):
    conversation_text = '\n'.join([CHATGLM_V2_PROMPT_TEMPLATE.format(prompt=utterance) for utterance in conversation_history])
    input_ids = tokenizer.encode(conversation_text, return_tensors="pt")

    with torch.inference_mode():
        output = model.generate(input_ids, max_length=1024)
        model_response = tokenizer.decode(output[0], skip_special_tokens=True)

    model_response = model_response[len(conversation_text):].strip()  # 删除生成结果中的 prompt

    return model_response


if __name__ == "__main__":
    while True:
        user_input = input("请输入：")  # 获取用户输入
        print(f"用户：{user_input}")

        # 将用户输入添加到聊天历史
        conversation_history.append(user_input)
        print(conversation_history)
        model_response = generate_response(conversation_history)
        print(f"机器人：{model_response}")

    
