from sqlalchemy import create_engine, Column, Integer, String, Text, ForeignKey
# from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, relationship
from sqlalchemy.types import JSON 
from sqlalchemy import Table, Column, String
from sqlalchemy import Table, Column, Integer, String, MetaData
from sqlalchemy.orm import declarative_base
import os
import json
# 创建数据库引擎（使用SQLite）
db_url = "sqlite:///DataBase/Main.db"  
engine = create_engine(db_url)

# 创建一个基本的ORM类
Base = declarative_base()


# 第一张表格：课程和视频关联
class CourseVideo(Base):
    __tablename__ = 'course_videos'

    id = Column(Integer, primary_key=True, autoincrement=True)
    course_id = Column(String, ForeignKey('courses.id'), nullable=False)
    video_id = Column(String, ForeignKey('videos.id'), nullable=False)

# 第二张表格：视频详细信息
class Video(Base):
    __tablename__ = 'videos'

    id = Column(Integer, primary_key=True,autoincrement=True)
    video_id = Column(String, unique=True, nullable=False)
    title = Column(String)
    node_json = Column(JSON)
    edge_json = Column(JSON)
    tree_json = Column(JSON)

    # 建立反向关系，用于查询课程关联的视频
    courses = relationship('Course', secondary='course_videos')

# 第三张表格：课程信息
class Course(Base):
    __tablename__ = 'courses'

    id = Column(Integer, primary_key=True,autoincrement=True)
    course_id = Column(String, unique=True, nullable=False)
    repository_url = Column(String)
    # 可以添加其他课程信息字段

#第四张表：联查表
class Phrase(Base):
    __tablename__ = 'phrases'

    id = Column(Integer, primary_key=True,autoincrement=True)
    text = Column(String(10))
    tag = Column(Integer, default=0)
    url_list = Column(JSON, default=list)

# 创建表格
Base.metadata.create_all(engine)

# 创建一个会话
Session = sessionmaker(bind=engine)
session = Session()



def create_video(video_id, title, node_json, edge_json, tree_json):
    new_video = Video(video_id=video_id, title=title, node_json=node_json, edge_json=edge_json, tree_json=tree_json)
    session.add(new_video)
    session.commit()

def create_course(course_id,repository_url):
    new_course = Course(course_id=course_id,repository_url=repository_url)
    session.add(new_course)
    session.commit()

# 创建视频与课程的关联记录
def link_course_to_video(course_id, video_id):
    course_video_relation = CourseVideo(course_id=course_id, video_id=video_id)
    session.add(course_video_relation)
    session.commit()

# 创建视频流量数据表
def create_video_traffic_table(video_id):
    metadata = MetaData()

    # 动态生成表名
    table_name = f'video_traffic_{video_id}'

    video_traffic = Table(
        table_name,
        metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('timestamp', Integer),
        Column('traffic', Integer)
    )

    metadata.create_all(engine)  # 创建新表

# 向视频流量数据表中插入数据
def insert_video_traffic_data(video_id, timestamp, traffic):
    metadata = MetaData()
    table_name = f'video_traffic_{video_id}'

    # 定义表结构
    video_traffic_table = Table(
        table_name,
        metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('timestamp', Integer),
        Column('traffic', Integer),
        autoload_with=engine,  # 加载已存在的表结构
        extend_existing=True   # 如果表已存在，则扩展其结构
    )

    # 插入数据
    insert_statement = video_traffic_table.insert().values(timestamp=timestamp, traffic=traffic)

    session = Session()
    try:
        # 使用会话执行插入
        session.execute(insert_statement)
        session.commit()  # 明确提交事务
        #print(f"已经成功插入数据到 {table_name}")
    except Exception as e:
        session.rollback()  # 出现错误时回滚事务
        print(f"数据插入错误: {e}")
    finally:
        session.close() 

def insert_data(table_class, data):
    try:
        # 查询是否已存在记录
        record = session.query(table_class).filter_by(**data).first()

        if record:
            # 如果记录已存在，更新它的字段值
            for key, value in data.items():
                setattr(record, key, value)
            session.commit()
            #print(f"成功更新数据：{data}")
        else:
            # 如果记录不存在，创建一个新记录
            new_record = table_class(**data)
            session.add(new_record)
            session.commit()
            #print(f"成功插入新数据：{data}")

    except Exception as e:
        session.rollback()
        print(f"插入/更新数据时出错：{str(e)}")

def inject_videos_and_link_to_course(course_id_name):
    base_path = os.path.dirname(os.path.abspath(__file__))
    node_json_folder = os.path.join(base_path, "DataBase/NodeJson")

    # 确保课程已存在，如果不存在则创建
    existing_course = session.query(Course).filter_by(course_id=course_id_name).first()
    if not existing_course:
        new_course = Course(course_id=course_id_name)
        session.add(new_course)
        session.commit()
        course_id = new_course.id
    else:
        course_id = existing_course.id

    for file_name in os.listdir(node_json_folder):
        if file_name.endswith('.json'):
            video_file_id = os.path.splitext(file_name)[0]  # 使用文件名作为video_id

            # 检查视频是否已存在，如果不存在则创建
            existing_video = session.query(Video).filter_by(video_id=video_file_id).first()
            if not existing_video:
                new_video = Video(video_id=video_file_id)
                session.add(new_video)
                session.commit()
                existing_video = new_video

            # 关联课程和视频
            new_course_video = CourseVideo(course_id=course_id, video_id=existing_video.video_id)
            session.add(new_course_video)
            session.commit()

def display_table_info(table_name):
    try:
        metadata = MetaData()
        metadata.bind = engine
        if table_name in ['CourseVideo', 'Video', 'Course']:
            # 对于静态定义的表
            table_class = globals()[table_name]
            rows = session.query(table_class).all()
            if not rows:
                print(f"{table_name} 表内没有数据")
            else:
                print(f"{table_name} 表内的数据:")
                for row in rows:
                    for column in row.__table__.columns:
                        print(f"{column.description}: {getattr(row, column.name)}")
                    print("-" * 20)
        else:
            # 对于动态创建的表
            table = Table(table_name, metadata, autoload_with=engine)
            with engine.connect() as connection:
                result = connection.execute(table.select())
                rows = result.fetchall()
                if not rows:
                    print(f"{table_name} 表内没有数据")
                else:
                    print(f"{table_name} 表内的数据:")
                    for row in rows:
                        print(row)  # 打印整行数据
                        print("-" * 20)
    except Exception as e:
        print(f"查询 {table_name} 表内的数据时出错:", str(e))

def read_json_file(file_path):
    with open(file_path, 'r', encoding='utf-8') as file:
        return json.load(file)

def inject_json_data_to_database():
    base_path = os.path.dirname(os.path.abspath(__file__))
    json_folders = ["DataBase/NodeJson", "DataBase/EdgeJson", "DataBase/TreeJson"]

    for file_name in os.listdir(os.path.join(base_path, json_folders[0])):
        if file_name.endswith('.json'):
            video_id = os.path.splitext(file_name)[0]

            node_json = read_json_file(os.path.join(base_path, json_folders[0], file_name))
            edge_json = read_json_file(os.path.join(base_path, json_folders[1], file_name))
            tree_json = read_json_file(os.path.join(base_path, json_folders[2], file_name))

            existing_video = session.query(Video).filter_by(video_id=video_id).first()
            if existing_video:
                # 更新现有记录
                existing_video.title = "title" + str(video_id) # 更新标题
                existing_video.node_json = node_json
                existing_video.edge_json = edge_json
                existing_video.tree_json = tree_json
            else:
                # 插入新记录
                new_video = Video(
                    video_id=video_id, 
                    title="title" + str(video_id),  # 提供标题
                    node_json=node_json, 
                    edge_json=edge_json, 
                    tree_json=tree_json
                )
                session.add(new_video)
            session.commit()

#以下是将所有视频标题更新为title+id，比如说title1，title2
def update_video_titles():
    # 查询所有视频记录
    videos = session.query(Video).all()

    # 遍历每个视频并更新其标题
    for video in videos:
        video.title = "title" + str(video.id)

    # 提交更改
    session.commit()
    #print("所有视频标题已更新。")

def insert_traffic_data(video_id, traffic_data):
    table_name = f'video_traffic_{video_id}'
    metadata = MetaData()
    video_traffic_table = Table(
        table_name,
        metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('timestamp', Integer),
        Column('traffic', Integer),
        autoload_with=engine,
        extend_existing=True
    )

    for i, traffic in enumerate(traffic_data, start=1):  # 使用 enumerate 从 1 开始计数
        insert_statement = video_traffic_table.insert().values(timestamp=i, traffic=traffic)
        session.execute(insert_statement)

    session.commit()

def clear_table(table_name):
    try:
        metadata = MetaData()
        metadata.bind = engine

        # 检查表名是否为动态创建的表
        if table_name.startswith('video_traffic_'):
            table = Table(table_name, metadata, autoload_with=engine)
        else:
            # 对于静态定义的表
            table_class = globals().get(table_name)
            if not table_class:
                print(f"无法识别的表格名称: {table_name}")
                return
            table = table_class.__table__

        # 删除表内的数据并提交事务
        with engine.connect() as connection:
            connection.execute(table.delete())
            connection.commit()  # 提交事务
            #print(f"{table_name} 表内的数据已成功清除")

    except Exception as e:
        print(f"清除 {table_name} 表内的数据时出错:", str(e))

#在新的视频转化后更新表
def update_phrase(new_word,video_url):
    #尝试从数据库中查找匹配的词
    video_entry = session.query(Phrase).filter_by(text=new_word).first()

    if video_entry:
        #print("Record exists:", video_entry.text)
        #print(video_url)
        video_entry.tag = 1
        if video_url not in video_entry.url_list:
            #video_entry.url_list.append(video_url)
            new_list = video_entry.url_list.copy()
            new_list.append(video_url)
            #print("new_list:",new_list)
            video_entry.url_list = new_list
        #print("list_url before commit:",video_entry.url_list)
        session.commit()
        session.refresh(video_entry)
        #print("list_url after commit:",video_entry.url_list)
        

        
    else:
        #print("Record does not exist. Creating new record.")
        new_entry = Phrase(text=new_word, tag=0, url_list=[video_url])
        session.add(new_entry)
        #print("new record url:",new_entry.url_list)
        #print("")
        session.commit()

#前端发送请求读取表
def search_phrase(tar_word):
    record = session.query(Phrase).filter_by(text=tar_word, tag=1).first()

    if record:
        # print(record.url_list)
        return record.url_list
    else:
        return None

#用url查导图
def get_video_info_by_url(url):
    try:
        video_info = session.query(Video).filter_by(video_id=url).first()

        if video_info:
            #print("successful")
            return{"title":video_info.title,"tree_json":video_info.tree_json}
        else:
            #print("fail")
            return None
    except Exception as e:
        print(f"查询数据时出错：{str(e)}")
        return None
    finally:
        session.close()